<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="UTF-8">
    <title>Test Email</title>
  </head>
  <body>
    <p>Welcome to django-aws-mail! This is a test email message.</p>
    <p>To find this package on pypi, click here:</p>
    <p><a href="https://pypi.org/project/django-aws-mail/" style="display:inline-block;background-color:#328547;color:#fff;border-radius:5px;padding:10px 30px;text-decoration:none;font-size:1.5rem;" target="_blank">Go to PyPi</a></p>
    <p style="color:#888">If this email isn't intended for you, simply discard it.</p>
  </body>
</html>
